#ifndef MATRIX4X4HEADER
#define MATRIX4X4HEADER
#include <stdio.h>
#include <assert.h>


class Matrix4x4;
class Vector3;

//#include "Vector3.h"

class Matrix4x4
{
	
	//The data in row major format.
	float	data[16];
	friend class Vector3;
		
public:
	Matrix4x4();
	Matrix4x4(const Matrix4x4& m);
	Matrix4x4(const float * data);

	~Matrix4x4();

	
	//load with the identity matrix
	void Identity();

	//print the matrix. This is C-style because I hate cout/cin . . .
	void Print(FILE * f) const;

	//transpose the matrix
	void Transpose();

	//this = m *  this
	void PreMult(const Matrix4x4& m);

	//this = this * m
	void PostMult(const Matrix4x4 & m);

	//this = m1*m2;
	void Mult(const Matrix4x4 & m1,const Matrix4x4 & m2);

	//this =sc*this;
	void Mult(const float sc);
	
	//construct an X rotation matrix
	void RotateX(const float angle);

	//construct an Y rotation matrix
	void RotateY(const float angle);

	//construct an Z rotation matrix
	void RotateZ(const float angle);

	//construct an XYZ rotation matrix
	void RotateXYZ(const float X,const float Y,const float Z);
	void RotateXYZ(const Vector3 & v);

	//construct a translation matrix
	void Translate(const float X,const float Y,const float Z);
	void Translate(const Vector3 & v);

	//construct a scale matrix
	void Scale(const float X,const float Y,const float Z);
	void Scale(const Vector3 & scale);

	//this +=m
	void Add(const Matrix4x4 & m);
	//this =m1+m2
	void Add(const Matrix4x4 & m1,const Matrix4x4 & m2);
	
	//this -=m
	void Sub(const Matrix4x4 & m);
	//this = m1-m2
	void Sub(const Matrix4x4 & m1,const Matrix4x4 & m2);

	
	//set parts of the matrix with float arrays
	void SetRow(const float* f, const unsigned int row);
	void SetCol(const float* f, const unsigned int col);
	void SetAll(const float* f);
	void inline Set(const float val, const unsigned int index);

	//copy-out functions
	void GetRow(float* f, const unsigned int row) const;
	void GetCol(float* f, const unsigned int col) const;
	void GetAll(float* f) const;
	float inline Get(const unsigned int index) const;

	const float * GetPtr() const;

	bool GetInverse(Matrix4x4 & m) const;

};

void inline
Matrix4x4::Set(const float val, const unsigned int index)
{
	assert(index <= 15);
	data[index] = val;
}

float inline
Matrix4x4::Get(const unsigned int index) const
{
	assert(index <= 15);
	return data[index];
}


#endif
