#ifndef POSEHEADER
#define POSEHEADER

#include <vector>
#include <Matrix4x4.h>
#include <Quaternion.h>
#include <Vector3.h>

class Pose
{
	std::vector<Quaternion> rots;
	Vector3 position;
	Quaternion heading;
	float height;
	
	unsigned int size;
public:
	Pose();
	Pose(const unsigned int numjoints);
	~Pose();
		
	unsigned int GetSize() const;
	
	void SetJointRot(const unsigned int joint, const Quaternion & q);
	Quaternion GetJointRot(const unsigned int joint) const;
	
	double Compare(const Pose & p) const;
	void LinearInterpolate(const Pose & p1, const Pose & p2, const float f);
	
	void GetPosition(Vector3 & v) const;
	void SetPosition(const Vector3 & v);
	void SetHeading(const Quaternion & q);
	void GetHeading(Quaternion & q) const;
	
	void AddTranslation(const Vector3 & v);
	void AddBaseRotation(const Quaternion & q);
	
	void SetHeight(const float h);
	float GetHeight();

};




#endif