#include "ActionSet.h"

ActionSet::ActionSet(const std::string act)
{
	action = act;
	matrixDirty = true;
}

ActionSet::~ActionSet()
{
}

ActionSet::ActionSet()
{
	matrixDirty = true;
}

void 
ActionSet::AddAnim(const std::string & name, std::map<std::string,float> & attribs)
{
	animlisttype::iterator animit = animlist.find(name);
	if (animit != animlist.end())
		return;

	animMap[(int)animMap.size()] = name;
	animlist[name] = attribs;
	for (std::map<std::string,float>::iterator i = attribs.begin(); i != attribs.end(); i++)
	{
		std::map<std::string,int>::iterator j = attribMap.find(i->first);
		if (j == attribMap.end())
			attribMap[i->first] = (int)attribMap.size();
	}
	matrixDirty = true;
}


void 
ActionSet::RebuildMatrix()
{
	matrix.resize(animlist.size()*attribMap.size(),0.0f);

	int animnum = 0;
	//for (animlisttype::iterator i = animlist.begin(); i != animlist.end(); i++)
	
	for (unsigned int i = 0; i < animMap.size(); i++)
	{
		animlisttype::iterator it = animlist.find(animMap[i]);
		for (attriblisttype::iterator j = it->second.begin(); j != it->second.end(); j++)
			matrix[animnum*attribMap.size() + attribMap[j->first]] = j->second;
		animnum++;
	}
	matrixDirty = false;
}

std::string 
ActionSet::PickAnim(const std::map<std::string,float> & softc)
{
	if (matrixDirty)
		RebuildMatrix();

	//build the contraint vector
	std::valarray<float> v;
	v.resize(attribMap.size(),0.0f);
	for (std::map<std::string,float>::const_iterator i = softc.begin(); i != softc.end(); i++)
	{
		std::map<std::string,int>::iterator j = attribMap.find(i->first);
		if (j == attribMap.end())
		{
			continue;
		}
		v[j->second] = i->second;
	}

	//now do a big ol' slicey matrix multiply
	std::valarray<float> result;
	result.resize(attribMap.size(),0);
	for (unsigned int k = 0; k < attribMap.size(); k++)
	{
		std::slice s(k*attribMap.size(),attribMap.size(),1);
		std::valarray<float> row = matrix[s];
		result[k] = (v* (std::valarray<float>)matrix[s]).sum();
	}
	

	if (result.size() <= 0)
		return animMap[0];

	unsigned int maxindex = 0;
	float max = result[maxindex];

	for (unsigned int k = 0; k < result.size(); k++)
	{
		if (result[k] > max)
		{
			max = result[k];
			maxindex = k;
		}
	}


	return animMap[maxindex];
}
