#include "BoxingSim.h"

BoxingSim::BoxingSim()
{
}

BoxingSim::~BoxingSim()
{
}

bool 
BoxingSim::Initialize()
{
	initTime = TimeStamp();

	theta = phi = 0;
	dist = 10;
	theCamera.SetDist(dist);
	theCamera.SetOrbitAngle(theta,phi);
	theCamera.Orbit();
	overheadLight.SetPos(0,20,0);
	frontLight.SetPos(0,0,5);
	frontLight.SetAmbient(.5,.5,.5);
	Font::LoadFont("Tahoma.bmp","tahoma.dat","Tahoma",false);
	updates = 0;

	//load agent actors
	boxer1.LoadActor("boxer.xml");
	boxer1.SetPosition(Vector3(0,0,2));
	boxer1.SetHeading(3.14159f);

	boxer2.LoadActor("boxer.xml");
	boxer2.SetPosition(Vector3(0,0,-2));


	boxer1.SetOpponent(&boxer2);
	boxer2.SetOpponent(&boxer1);

	return true;
}

void 
BoxingSim::RenderPersp()
{
	
	
	TimeStamp t1;
	glEnable(GL_LIGHTING);
	
	//look through the default camera
	theCamera.SetView();

	frontLight.Switch(true);
	overheadLight.Switch(true);

	//draw the world coordinate axis
	glPushMatrix(); glTranslatef(10,0,10);
	Axis3D::StaticDraw(); glPopMatrix();

	glPushMatrix(); glTranslatef(-10,0,10);
	Axis3D::StaticDraw(); glPopMatrix();

	glPushMatrix(); glTranslatef(10,0,-10);
	Axis3D::StaticDraw(); glPopMatrix();

	glPushMatrix(); glTranslatef(-10,0,-10);
	Axis3D::StaticDraw(); glPopMatrix();

	glColor3f(.3f,.3f,.3f);
	glBegin(GL_QUADS);
	glNormal3f(0,1,0);
	glVertex3f(-10,0,10);
	glVertex3f(10,0,10);
	glVertex3f(10,0,-10);
	glVertex3f(-10,0,-10);
	glEnd();

	boxer1.Render();
	boxer2.Render();


	glDisable(GL_LIGHTING);
	renderTime = t1.Interval(TimeStamp());


}

void 
BoxingSim::RenderOrtho()
{
	static TimeStamp last;
	
	glTranslatef(10,24,0);
	glColor3f(1,1,1);

	Font::Printf("Tahoma",30,false,"FPS:%3.2f",1.0f/last.Interval(TimeStamp()));

	glLoadIdentity();
	glTranslatef(0,(float)windowY-30*4,0);
	Font::Printf("Tahoma",30,false,"Boxer 1");
	glTranslatef(0,30,0);
	for (unsigned int i = 0; i < 3; i++)
	{
		Font::Printf("Tahoma",30,false,boxer1.GetThoughts(i).c_str());
		glTranslatef(0,30,0);
	}

	glLoadIdentity();
	glTranslatef((float)windowX-200,(float)windowY-30*4,0);
	Font::Printf("Tahoma",30,false,"Boxer 2");
	glTranslatef(0,30,0);
	for (unsigned int i = 0; i < 3; i++)
	{
		Font::Printf("Tahoma",30,false,boxer2.GetThoughts(i).c_str());
		glTranslatef(0,30,0);
	}

	last.Reset();


}

void
BoxingSim::HandleInput()
{
	std::set<InputEvent>::iterator it;
	for (it = events.begin(); it != events.end(); it++)
	{
		switch(it->type)
		{
		case InputEvent::EventType::KeyDown:
			switch(it->ID)
			{
			case 'q':
				exit(1);
			break;

			case '1':
			case '2':
				boxer1.AddInputEvent(*it);
				break;
			case '3':
			case '4':
				boxer2.AddInputEvent(*it);
				break;
			}
			break;
		case InputEvent::EventType::MouseDrag:
			theta += it->coordinate.Get(0)*.01f;
			phi += it->coordinate.Get(1)*.01f;
			dist += it->coordinate.Get(2)*.1f;
			theCamera.SetOrbitAngle(theta,phi);
			theCamera.SetDist(dist);
			theCamera.Orbit();
			break;

		default:
			break;

		}
	}
	events.clear();
}


void 
BoxingSim::Update()
{

	HandleInput();

	TimeStamp t1;
	interval = lastUpdate.Interval(t1);
	lastUpdate = t1;
	updates++;

	boxer1.Think();
	boxer2.Think();

	boxer1.UpdateAnim();
	boxer2.UpdateAnim();
	
	theCamera.SetOrbitAngle((float)t1.Msec()/1000.0f,1.0f );
	Vector3 lfv,rfv,bp,pv;
	Matrix4x4 lfm,rfm,pm;
	
	updateTime = t1.Interval(TimeStamp());
}

