#include "Matrix4x4.h"
#include "Vector3.h"
#include <memory.h>
#include <math.h>

#define swap(a,b,temp) temp=a; a=b; b=temp;

Matrix4x4::Matrix4x4()
{
	Identity();
}

//copy constructor
Matrix4x4::Matrix4x4(const Matrix4x4& m)
{
	memcpy(data, m.data,sizeof(data));
}

Matrix4x4::Matrix4x4(const float * dat)
{
	memcpy(data, dat,sizeof(data));
}

Matrix4x4::~Matrix4x4()
{
}


const float *
Matrix4x4::GetPtr() const
{
	return data;
}
void
Matrix4x4::Identity()
{
	memset(data,0,sizeof(data));
	data[0] = data[5] = data[10] = data[15] = 1.0f;
}

void
Matrix4x4::Print(FILE * f)const
{
	fprintf(f,"|%.2f\t%.2f\t%.2f\t%.2f|\n",data[0],data[1],data[2],data[3]);
	fprintf(f,"|%.2f\t%.2f\t%.2f\t%.2f|\n",data[4],data[5],data[6],data[7]);
	fprintf(f,"|%.2f\t%.2f\t%.2f\t%.2f|\n",data[8],data[9],data[10],data[11]);
	fprintf(f,"|%.2f\t%.2f\t%.2f\t%.2f|\n",data[12],data[13],data[14],data[15]);
}

//m * this
void 
Matrix4x4::PreMult(const Matrix4x4& m)
{
	Matrix4x4 temp(data);
    data[ 0] = m.data[ 0] * temp.data[ 0] + m.data[ 1] * temp.data[ 4] + m.data[ 2] * temp.data[ 8] + m.data[ 3] * temp.data[12];
    data[ 1] = m.data[ 0] * temp.data[ 1] + m.data[ 1] * temp.data[ 5] + m.data[ 2] * temp.data[ 9] + m.data[ 3] * temp.data[13];
    data[ 2] = m.data[ 0] * temp.data[ 2] + m.data[ 1] * temp.data[ 6] + m.data[ 2] * temp.data[10] + m.data[ 3] * temp.data[14];
    data[ 3] = m.data[ 0] * temp.data[ 3] + m.data[ 1] * temp.data[ 7] + m.data[ 2] * temp.data[11] + m.data[ 3] * temp.data[15];

    data[ 4] = m.data[ 4] * temp.data[ 0] + m.data[ 5] * temp.data[ 4] + m.data[ 6] * temp.data[ 8] + m.data[ 7] * temp.data[12];
    data[ 5] = m.data[ 4] * temp.data[ 1] + m.data[ 5] * temp.data[ 5] + m.data[ 6] * temp.data[ 9] + m.data[ 7] * temp.data[13];
    data[ 6] = m.data[ 4] * temp.data[ 2] + m.data[ 5] * temp.data[ 6] + m.data[ 6] * temp.data[10] + m.data[ 7] * temp.data[14];
    data[ 7] = m.data[ 4] * temp.data[ 3] + m.data[ 5] * temp.data[ 7] + m.data[ 6] * temp.data[11] + m.data[ 7] * temp.data[15];

    data[ 8] = m.data[ 8] * temp.data[ 0] + m.data[ 9] * temp.data[ 4] + m.data[10] * temp.data[ 8] + m.data[11] * temp.data[12];
    data[ 9] = m.data[ 8] * temp.data[ 1] + m.data[ 9] * temp.data[ 5] + m.data[10] * temp.data[ 9] + m.data[11] * temp.data[13];
    data[10] = m.data[ 8] * temp.data[ 2] + m.data[ 9] * temp.data[ 6] + m.data[10] * temp.data[10] + m.data[11] * temp.data[14];
    data[11] = m.data[ 8] * temp.data[ 3] + m.data[ 9] * temp.data[ 7] + m.data[10] * temp.data[11] + m.data[11] * temp.data[15];

    data[12] = m.data[12] * temp.data[ 0] + m.data[13] * temp.data[ 4] + m.data[14] * temp.data[ 8] + m.data[15] * temp.data[12];
    data[13] = m.data[12] * temp.data[ 1] + m.data[13] * temp.data[ 5] + m.data[14] * temp.data[ 9] + m.data[15] * temp.data[13];
    data[14] = m.data[12] * temp.data[ 2] + m.data[13] * temp.data[ 6] + m.data[14] * temp.data[10] + m.data[15] * temp.data[14];
    data[15] = m.data[12] * temp.data[ 3] + m.data[13] * temp.data[ 7] + m.data[14] * temp.data[11] + m.data[15] * temp.data[15];
	 
}

//this*m
void 
Matrix4x4::PostMult(const Matrix4x4 & m)
{
	Matrix4x4 temp(data);
    data[ 0] = temp.data[ 0] * m.data[ 0] + temp.data[ 1] * m.data[ 4] + temp.data[ 2] * m.data[ 8] + temp.data[ 3] * m.data[12];
    data[ 1] = temp.data[ 0] * m.data[ 1] + temp.data[ 1] * m.data[ 5] + temp.data[ 2] * m.data[ 9] + temp.data[ 3] * m.data[13];
    data[ 2] = temp.data[ 0] * m.data[ 2] + temp.data[ 1] * m.data[ 6] + temp.data[ 2] * m.data[10] + temp.data[ 3] * m.data[14];
    data[ 3] = temp.data[ 0] * m.data[ 3] + temp.data[ 1] * m.data[ 7] + temp.data[ 2] * m.data[11] + temp.data[ 3] * m.data[15];

    data[ 4] = temp.data[ 4] * m.data[ 0] + temp.data[ 5] * m.data[ 4] + temp.data[ 6] * m.data[ 8] + temp.data[ 7] * m.data[12];
    data[ 5] = temp.data[ 4] * m.data[ 1] + temp.data[ 5] * m.data[ 5] + temp.data[ 6] * m.data[ 9] + temp.data[ 7] * m.data[13];
    data[ 6] = temp.data[ 4] * m.data[ 2] + temp.data[ 5] * m.data[ 6] + temp.data[ 6] * m.data[10] + temp.data[ 7] * m.data[14];
    data[ 7] = temp.data[ 4] * m.data[ 3] + temp.data[ 5] * m.data[ 7] + temp.data[ 6] * m.data[11] + temp.data[ 7] * m.data[15];

    data[ 8] = temp.data[ 8] * m.data[ 0] + temp.data[ 9] * m.data[ 4] + temp.data[10] * m.data[ 8] + temp.data[11] * m.data[12];
    data[ 9] = temp.data[ 8] * m.data[ 1] + temp.data[ 9] * m.data[ 5] + temp.data[10] * m.data[ 9] + temp.data[11] * m.data[13];
    data[10] = temp.data[ 8] * m.data[ 2] + temp.data[ 9] * m.data[ 6] + temp.data[10] * m.data[10] + temp.data[11] * m.data[14];
    data[11] = temp.data[ 8] * m.data[ 3] + temp.data[ 9] * m.data[ 7] + temp.data[10] * m.data[11] + temp.data[11] * m.data[15];

    data[12] = temp.data[12] * m.data[ 0] + temp.data[13] * m.data[ 4] + temp.data[14] * m.data[ 8] + temp.data[15] * m.data[12];
    data[13] = temp.data[12] * m.data[ 1] + temp.data[13] * m.data[ 5] + temp.data[14] * m.data[ 9] + temp.data[15] * m.data[13];
    data[14] = temp.data[12] * m.data[ 2] + temp.data[13] * m.data[ 6] + temp.data[14] * m.data[10] + temp.data[15] * m.data[14];
    data[15] = temp.data[12] * m.data[ 3] + temp.data[13] * m.data[ 7] + temp.data[14] * m.data[11] + temp.data[15] * m.data[15];

}

void 
Matrix4x4::Mult(const Matrix4x4 & m1,const Matrix4x4 & m2)
{
    data[ 0] = m1.data[ 0] * m2.data[ 0] + m1.data[ 1] * m2.data[ 4] + m1.data[ 2] * m2.data[ 8] + m1.data[ 3] * m2.data[12];
    data[ 1] = m1.data[ 0] * m2.data[ 1] + m1.data[ 1] * m2.data[ 5] + m1.data[ 2] * m2.data[ 9] + m1.data[ 3] * m2.data[13];
    data[ 2] = m1.data[ 0] * m2.data[ 2] + m1.data[ 1] * m2.data[ 6] + m1.data[ 2] * m2.data[10] + m1.data[ 3] * m2.data[14];
    data[ 3] = m1.data[ 0] * m2.data[ 3] + m1.data[ 1] * m2.data[ 7] + m1.data[ 2] * m2.data[11] + m1.data[ 3] * m2.data[15];

    data[ 4] = m1.data[ 4] * m2.data[ 0] + m1.data[ 5] * m2.data[ 4] + m1.data[ 6] * m2.data[ 8] + m1.data[ 7] * m2.data[12];
    data[ 5] = m1.data[ 4] * m2.data[ 1] + m1.data[ 5] * m2.data[ 5] + m1.data[ 6] * m2.data[ 9] + m1.data[ 7] * m2.data[13];
    data[ 6] = m1.data[ 4] * m2.data[ 2] + m1.data[ 5] * m2.data[ 6] + m1.data[ 6] * m2.data[10] + m1.data[ 7] * m2.data[14];
    data[ 7] = m1.data[ 4] * m2.data[ 3] + m1.data[ 5] * m2.data[ 7] + m1.data[ 6] * m2.data[11] + m1.data[ 7] * m2.data[15];

    data[ 8] = m1.data[ 8] * m2.data[ 0] + m1.data[ 9] * m2.data[ 4] + m1.data[10] * m2.data[ 8] + m1.data[11] * m2.data[12];
    data[ 9] = m1.data[ 8] * m2.data[ 1] + m1.data[ 9] * m2.data[ 5] + m1.data[10] * m2.data[ 9] + m1.data[11] * m2.data[13];
    data[10] = m1.data[ 8] * m2.data[ 2] + m1.data[ 9] * m2.data[ 6] + m1.data[10] * m2.data[10] + m1.data[11] * m2.data[14];
    data[11] = m1.data[ 8] * m2.data[ 3] + m1.data[ 9] * m2.data[ 7] + m1.data[10] * m2.data[11] + m1.data[11] * m2.data[15];

    data[12] = m1.data[12] * m2.data[ 0] + m1.data[13] * m2.data[ 4] + m1.data[14] * m2.data[ 8] + m1.data[15] * m2.data[12];
    data[13] = m1.data[12] * m2.data[ 1] + m1.data[13] * m2.data[ 5] + m1.data[14] * m2.data[ 9] + m1.data[15] * m2.data[13];
    data[14] = m1.data[12] * m2.data[ 2] + m1.data[13] * m2.data[ 6] + m1.data[14] * m2.data[10] + m1.data[15] * m2.data[14];
    data[15] = m1.data[12] * m2.data[ 3] + m1.data[13] * m2.data[ 7] + m1.data[14] * m2.data[11] + m1.data[15] * m2.data[15];

}

void 
Matrix4x4::Mult(const float sc)
{
	for (int i = 0; i < 16; i++)
		data[i] *= sc;
}

void 
Matrix4x4::SetRow(const float* f, const unsigned int row)
{
	if (row > 3)
		return;
	memcpy(&data[row], f,sizeof(float)*4);
}

void 
Matrix4x4::SetCol(const float* f, const unsigned int col)
{
	if (col > 3)
		return;
	data[col] = f[0];
	data[col+4] = f[1];
	data[col+8] = f[2];
	data[col+12] = f[3];
}

void 
Matrix4x4::SetAll(const float* f)
{
	memcpy(data, f,sizeof(float)*16);
}

void 
Matrix4x4::GetRow(float* f, const unsigned int row) const
{
	assert(row <= 3);
	memcpy(f,&data[row], sizeof(float)*4);
}

void 
Matrix4x4::GetCol(float* f, const unsigned int col) const
{
	assert(col <= 3);
	f[0] = data[col];
	f[1] = data[col+4];
	f[2] = data[col+8];
	f[3] = data[col+12];
}

void 
Matrix4x4::GetAll(float* f) const
{
	memcpy(f,data,sizeof(float)*16);
}

void
Matrix4x4::Transpose()
{

	float t;
	swap(data[1],data[4],t);
	swap(data[2],data[8],t);
	swap(data[3],data[12],t);
	swap(data[6],data[9],t);
	swap(data[7],data[13],t);
	swap(data[11],data[14],t);
}

void 
Matrix4x4::RotateX(const float angle)
{
    data[1] = data[2] = data[3] = data[4] = data[7] = data[8] = data[11] = data[12] = data[13] = data[14] = 0.0f;
    data[0] = data[15] = 1.0f;
    data[5] = data[10] = cosf(angle);
    data[9] = sinf(angle);
    data[6] = -data[9];    

}

void 
Matrix4x4::RotateY(const float angle)
{
    data[1] = data[3] = data[4] = data[6] = data[7] = data[9] = data[11] = data[12] = data[13] = data[14] = 0.0f;
    data[0] = data[10] = cosf(angle);
    data[2] = sinf(angle);
    data[8] = -data[2];
    data[5] = data[15] = 1.0f;

}
void 
Matrix4x4::RotateZ(const float angle)
{
    data[2] = data[3] = data[6] = data[7] = data[8] = data[9] = data[11] = data[12] = data[13] = data[14] = 0.0f;
    data[0] = data[5] = cosf(angle);
    data[1] = -sinf(angle);
    data[4] = -data[1];
    data[10] = data[15] = 1.0f;

}

void 
Matrix4x4::RotateXYZ(const Vector3 & v)
{
	RotateXYZ(v.data[0],v.data[1],v.data[2]);
}

void
Matrix4x4::RotateXYZ(const float X,const float Y,const float Z)
{
	Matrix4x4 rX, rY, rZ;

	//should really work this all out
	rX.RotateX(X);
	rY.RotateY(Y);
	rZ.RotateZ(Z);
	Identity();
	PostMult(rZ);
	PostMult(rY);
	PostMult(rX);
}

void
Matrix4x4::Translate(const float X,const float Y,const float Z)
{
	Identity();
	data[3] = X;
	data[7] = Y;
	data[11] = Z;
}

void 
Matrix4x4::Translate(const Vector3 & v)
{
	Identity();
	data[3] = v.data[0];
	data[7] = v.data[1];
	data[11] = v.data[2];
}

void 
Matrix4x4::Scale(const float X,const float Y,const float Z)
{
	memset(data,0,sizeof(data));
	data[0] = X;
	data[5] = Y;
	data[10] =Z;
	data[15] = 1.0f;
}

void 
Matrix4x4::Scale(const Vector3 & scale)
{
	memset(data,0,sizeof(data));
	data[0] = scale.data[0];
	data[5] = scale.data[1];
	data[10] =scale.data[2];
	data[15] = 1.0f;
	
}

void 
Matrix4x4::Add(const Matrix4x4 & m)
{
	for (int i = 0; i < 15;i++)
		data[i] += m.data[i];
}

void 
Matrix4x4::Add(const Matrix4x4 & m1,const Matrix4x4 & m2)
{
	for (int i = 0; i < 15;i++)
		data[i] = m1.data[i]+m2.data[i];
}

void 
Matrix4x4::Sub(const Matrix4x4 & m)
{
	for (int i = 0; i < 15;i++)
		data[i] -= m.data[i];
}

void 
Matrix4x4::Sub(const Matrix4x4 & m1,const Matrix4x4 & m2)
{
	for (int i = 0; i < 15;i++)
		data[i] = m1.data[i]-m2.data[i];
}


bool
Matrix4x4::GetInverse(Matrix4x4 & m) const
{
	//calculate the adjoint
	m.data[ 0] =  (data[ 5]*(data[10]*data[15]-data[11]*data[14])-data[ 6]*(data[ 9]*data[15]-data[11]*data[13])+data[ 7]*(data[ 9]*data[14]-data[10]*data[13]));
	m.data[ 4] = -(data[ 4]*(data[10]*data[15]-data[11]*data[14])-data[ 6]*(data[ 8]*data[15]-data[11]*data[12])+data[ 7]*(data[ 8]*data[14]-data[10]*data[12]));
	m.data[ 8] =  (data[ 4]*(data[ 9]*data[15]-data[11]*data[13])-data[ 5]*(data[ 8]*data[15]-data[11]*data[13])+data[ 7]*(data[ 8]*data[13]-data[ 9]*data[12]));
	m.data[12] = -(data[ 4]*(data[ 9]*data[14]-data[10]*data[13])-data[ 5]*(data[ 8]*data[14]-data[10]*data[12])+data[ 6]*(data[ 8]*data[13]-data[ 9]*data[12]));

	m.data[ 1] = -(data[ 1]*(data[10]*data[15]-data[11]*data[14])-data[ 2]*(data[ 9]*data[15]-data[11]*data[13])+data[ 3]*(data[ 9]*data[14]-data[10]*data[13]));
	m.data[ 5] =  (data[ 0]*(data[10]*data[15]-data[11]*data[14])-data[ 2]*(data[ 8]*data[15]-data[11]*data[12])+data[ 3]*(data[ 8]*data[14]-data[10]*data[12]));
	m.data[ 9] = -(data[ 0]*(data[ 9]*data[15]-data[11]*data[13])-data[ 1]*(data[ 8]*data[15]-data[11]*data[13])+data[ 3]*(data[ 8]*data[13]-data[ 9]*data[12]));
	m.data[13] =  (data[ 0]*(data[ 9]*data[14]-data[10]*data[13])-data[ 1]*(data[ 8]*data[14]-data[10]*data[12])+data[ 2]*(data[ 8]*data[13]-data[ 9]*data[12]));
	
	m.data[ 2] =  (data[ 1]*(data[ 6]*data[15]-data[ 7]*data[14])-data[ 2]*(data[ 5]*data[15]-data[ 7]*data[13])+data[ 3]*(data[ 5]*data[14]-data[ 6]*data[13]));
	m.data[ 6] = -(data[ 0]*(data[ 6]*data[15]-data[ 7]*data[14])-data[ 2]*(data[ 4]*data[15]-data[ 7]*data[12])+data[ 3]*(data[ 4]*data[14]-data[ 6]*data[12]));
	m.data[10] =  (data[ 0]*(data[ 5]*data[15]-data[ 7]*data[13])-data[ 1]*(data[ 4]*data[15]-data[ 7]*data[13])+data[ 3]*(data[ 4]*data[13]-data[ 5]*data[12]));
	m.data[14] = -(data[ 0]*(data[ 5]*data[14]-data[ 6]*data[13])-data[ 1]*(data[ 4]*data[14]-data[ 6]*data[12])+data[ 2]*(data[ 4]*data[13]-data[ 5]*data[12]));
	
	m.data[ 3] = -(data[ 1]*(data[ 6]*data[11]-data[ 7]*data[10])-data[ 2]*(data[ 5]*data[11]-data[ 7]*data[ 9])+data[ 3]*(data[ 5]*data[10]-data[ 6]*data[ 9]));
	m.data[ 7] =  (data[ 0]*(data[ 6]*data[11]-data[ 7]*data[10])-data[ 2]*(data[ 4]*data[11]-data[ 7]*data[ 8])+data[ 3]*(data[ 4]*data[10]-data[ 6]*data[ 8]));
	m.data[11] = -(data[ 0]*(data[ 5]*data[11]-data[ 7]*data[ 9])-data[ 1]*(data[ 4]*data[11]-data[ 7]*data[ 9])+data[ 3]*(data[ 4]*data[ 9]-data[ 5]*data[ 8]));
	m.data[15] =  (data[ 0]*(data[ 5]*data[10]-data[ 6]*data[ 9])-data[ 1]*(data[ 4]*data[10]-data[ 6]*data[ 8])+data[ 2]*(data[ 4]*data[ 9]-data[ 5]*data[ 8]));
	
	float determinate = data[0]* m.data[0] + data[1]*m.data[4] + data[2]*m.data[8] + data[3]*m.data[12];
	if (determinate < .0000000000000001)
		return false;
	
	float invdeterminate = 1/determinate;

	m.Mult(invdeterminate);

	return true;
}