#include <Pose.h>

Pose::Pose()
{
	height = 0;
	size = 0;
}

Pose::Pose(const unsigned int numjoints)
{
	height = 0;
	size = numjoints;
	rots.resize(numjoints);
}

Pose::~Pose()
{
}
		
unsigned int 
Pose::GetSize() const
{
	return size;
}

void 
Pose::SetHeight(const float h)
{
	height = h;
}

float 
Pose::GetHeight()
{
	return height;
}


void 
Pose::SetJointRot(const unsigned int joint, const Quaternion & q)
{
	if (joint >= size)
		return;
	rots[joint] = q;
}


Quaternion 
Pose::GetJointRot(const unsigned int joint) const
{
	if (joint >= size)
		return Quaternion();
	return rots[joint];
}


double 
Pose::Compare(const Pose & p) const
{
	//fix me!
	return 1.0f;
}


void 
Pose::AddTranslation(const Vector3 & v)
{
	position.Add(v);
}

void
Pose::AddBaseRotation(const Quaternion & q)
{
	heading = q*heading;
}

void 
Pose::LinearInterpolate(const Pose & p1, const Pose & p2, const float f)
{
	if ((p1.size != p2.size))
		return;

	if (p1.size != size)
	{
		size = p1.size;
		rots.resize(size);
	}

	for (unsigned int i = 0; i < size; i++)
		rots[i].slerp(p1.rots[i],p2.rots[i],f);
	
	position.Interpolate(p1.position,p2.position,f);
	heading.slerp(p1.heading,p2.heading,f);
	height = (1.0f-f)*p1.height+ f*p2.height;
	
}

void 
Pose::GetPosition(Vector3 & v) const
{
	v = position;
}

void 
Pose::SetPosition(const Vector3 & v)
{
	position = v;
}

void 
Pose::GetHeading(Quaternion & q) const
{
	q = heading;
}

void 
Pose::SetHeading(const Quaternion & q) 
{
	heading = q;
}

