#include "SkelChar.h"


int SkelChar::GUID = 0;
std::map<std::string,SkelChar*> SkelChar::skels;

SkelChar::SkelChar()
{
	numVerts = 0;
	numPolys = 0;
	numJoints = 0;
	numNormals = 0;
	numUVs = 0;
	joints.clear();
	verts.clear();
	polys.clear();
	norms.clear();
	uvs.clear();

}

SkelChar::~SkelChar()
{
	std::map<int,SkelCharInst*>::iterator it;
	for (it = instances.begin(); it != instances.end(); it++)
		delete it->second;

}


bool
SkelChar::Load(const char * name, const char * file)
{
	std::map<std::string,SkelChar*>::iterator i = skels.find(name);
	if(i != skels.end())
	{
		if (i->first == std::string(name))
			return true;
		else
		{
			printf("Attempted to load different skeleton files with the same name %s\n",name);
			return false;
		}
	}

	SkelChar * sk = new SkelChar();
	if (!sk->LoadSkel(file))
	{
		delete sk;
		return false;
	}
	
	skels[name] = sk;
	return true;
}


int 
SkelChar::JointIndex(const char * name)
{
	std::map<std::string, unsigned int>::iterator i = indexLUT.find(name);
	if (i != indexLUT.end())
		return i->second;
	return -1;
}

bool 
SkelChar::LoadSkel(const char * filename)
{
	FILE * f = fopen(filename,"rb");
	if (!f)
		return false;
	char header[5];
	fread(header,5,1,f);
	header[4] = 0;
	if (strncmp(header,"SKEL",4))
	{
		fclose(f);
		return false;
	}


	indexLUT.clear();
	//get the number of each element
	fread(&numJoints,4,1,f);
	fread(&numVerts,4,1,f);
	fread(&numPolys,4,1,f);
	fread(&numNormals,4,1,f);
	fread(&numUVs,4,1,f);

	joints.resize(numJoints);
	for (unsigned int i = 0; i < numJoints;i++)
	{
		float temp[3];
		char name[21];
		fread(name,1,21,f);
		joints[i].name = name;
		
		indexLUT[name] = i;

		fread(temp,sizeof(temp),1,f);
		joints[i].relpos.Set(temp);

		fread(temp,sizeof(temp),1,f);
		joints[i].orient.Set(temp);
		
		fread(&joints[i].parent,4,1,f);
		float mat[16];
		fread(mat,4,16,f);
		joints[i].MbindInv.SetAll(mat);
		joints[i].MbindInv.Transpose();
	}

	
	joints[0].relpos.Set(0,0,0); //move the root to the origin?
	
	
	
	verts.resize(numVerts);
	for (unsigned int i = 0; i < numVerts;i++)
	{
		float temp[3];
		fread(temp,4,3,f);
		verts[i].bindpos.Set(temp);
		//verts[i].bindpos.Mult(.413);
		//verts[i].bindpos.Add(-.001,1.885,-.045);
		fread(&verts[i].numjoints,4,1,f);

		verts[i].bone.resize(verts[i].numjoints);
		verts[i].weight.resize(verts[i].numjoints);
		verts[i].relpos.resize(verts[i].numjoints);

		for (unsigned int j = 0; j < verts[i].numjoints; j++)
			fread(&verts[i].bone[j],4,1,f);

		for (unsigned int j = 0; j < verts[i].numjoints; j++)
			fread(&verts[i].weight[j],4,1,f);
	}

	polys.resize(numPolys);
	for (unsigned int i = 0; i < numPolys;i++)
	{
		fread(polys[i].verts,4,3,f);
		fread(polys[i].norms,4,3,f);
		fread(polys[i].uvs,4,3,f);
	}

	norms.resize(numNormals);
	for (unsigned int i = 0; i < numNormals;i++)
	{
		float temp[3];
		fread(temp,4,3,f);
		norms[i].orient.Set(temp);
		norms[i].vertex = -1;
	}

	uvs.resize(numUVs);
	for (unsigned int i = 0; i < numUVs;i++)
	{
		fread(uvs[i].dat,4,2,f);
	}

	fclose(f);


	//set up the normals. This will update each shared normal more than once, but oh well...
	for (unsigned int i = 0; i < numPolys; i++)
	{
		norms[polys[i].norms[0]].relorient.resize(verts[polys[i].verts[0]].numjoints);
		norms[polys[i].norms[1]].relorient.resize(verts[polys[i].verts[1]].numjoints);
		norms[polys[i].norms[2]].relorient.resize(verts[polys[i].verts[2]].numjoints);
		norms[polys[i].norms[0]].vertex = polys[i].verts[0];
		norms[polys[i].norms[1]].vertex = polys[i].verts[1];
		norms[polys[i].norms[2]].vertex = polys[i].verts[2];
	}


	//set up the skeleton

	//first update the bone matrices
	for (unsigned int i = 0; i < numJoints;i++)
	{

		Matrix4x4 trans;
		Matrix4x4 rotate;

		//transform from parent coordinate frame to local
		rotate.RotateXYZ(joints[i].orient);
		trans.Translate(joints[i].relpos);
		joints[i].Mrelative.Mult(trans,rotate);

		//get the inverse
		rotate.Transpose();
		trans.Translate(-1.0f *joints[i].relpos);
		joints[i].Minverse.Mult(rotate,trans);


		//contatentate the inverses to get the absolute inverse
		
		if (joints[i].parent >= 0)
			joints[i].Minverse.PostMult(joints[joints[i].parent].Minverse);

		//concatenate the forward transforms to get the absolute
		if (joints[i].parent >= 0)
			joints[i].Mabsolute.Mult(joints[joints[i].parent].Mabsolute,joints[i].Mrelative);
		else
			joints[i].Mabsolute = joints[i].Mrelative;
	}


	//calculate the vertices relative positions
	for (unsigned int i = 0; i < numVerts; i++)
		for (unsigned int j = 0; j < verts[i].numjoints; j++)
		{
			
			verts[i].relpos[j].Mult(verts[i].bindpos, joints[verts[i].bone[j]].MbindInv);
		}

	//calculate the normals relative positions
	for (unsigned int i = 0; i < numNormals; i++)
	{
		if (norms[i].vertex < 0)
			continue;

		for (unsigned int j = 0; j < verts[norms[i].vertex].numjoints; j++)
		{
			norms[i].relorient[j].MultRot(norms[i].orient, joints[verts[norms[i].vertex].bone[j]].MbindInv);
		}
	}

	return true;
}

int
SkelChar::CreateInstance(const char * name)
{
	std::map<std::string,SkelChar*>::iterator it;
	it =skels.find(name) ; 
	if (it == skels.end())
		return -1;

	skels[name]->instances[GUID] = new SkelCharInst(*it->second,false);
	
	GUID++;
	
	return GUID-1;
}

 SkelCharInst * 
SkelChar::GetInstancePtr(const char * name, const unsigned int inst)
{
	std::map<std::string,SkelChar*>::iterator it;

	it = skels.find(name);
	if (it == skels.end())
		return NULL;

	std::map<int,SkelCharInst*>::iterator it2;
	it2 = it->second->instances.find(inst);

	if ( it2 == it->second->instances.end())
		return NULL;
	
	return it2->second;
}


 void 
SkelChar::Unload(const char * name)
{
	std::map<std::string,SkelChar*>::iterator it;
	it = skels.find(name);
	if (it == skels.end())
		return;

	
	delete it->second;

	skels.erase(it);
}


 void 
SkelChar::RetireInstance(const char * name, const unsigned int inst)
{
	std::map<std::string,SkelChar*>::iterator it;

	it = skels.find(name);
	if (it == skels.end())
		return;

	std::map<int,SkelCharInst*>::iterator it2;
	it2 = it->second->instances.find(inst);

	if ( it2 == it->second->instances.end())
		return;
	delete it2->second;
	it->second->instances.erase(it2);
}
