//this is for win32's benefit (I do most of my coding at home)

#include <sdl.h>
#include <windows.h>
#include <GL/gl.h> 
#include <GL/glu.h> 
#include <stdlib.h>

#include <Mmsystem.h>
#include <stdio.h> 
#include <math.h>
#include "TestScene.h"
#include "BoxingSim.h"

#include "TimeStamp.h"

/* window size constants*/ 
const int XSIZE = 1024; 
const int YSIZE = 768; 
const bool fullScreen = false;

BoxingSim theScene;
TimeStamp last = TimeStamp();
//double minUpdate = 1.0f/55.0f;


//mouse states
bool LB,MB,RB;
int dx,dy;


void RenderScene(void) 
{ 

	TimeStamp now = TimeStamp();
//	if (last.Interval(now) < minUpdate)
//		return;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 
	static double obj = 0;

	//set up the projection matrix for perspective
	glMatrixMode(GL_PROJECTION); 
	glLoadIdentity();
	gluPerspective(45,(GLfloat)XSIZE/(GLfloat)YSIZE,.1,1000);

	//initialze the modelview matrix to identity
	glMatrixMode(GL_MODELVIEW); 
	glLoadIdentity(); 

	//render the scene's perspective components
	theScene.RenderPersp();


	//draw the info text in ortho mode
	glMatrixMode(GL_PROJECTION); 
	glLoadIdentity();
	glOrtho(0,XSIZE,YSIZE,0,-1,1);

	glMatrixMode(GL_MODELVIEW); 
	glLoadIdentity();

	//render the scene's ortho components
	theScene.RenderOrtho();


	glFlush(); 
	SDL_GL_SwapBuffers();	
	last = now;
} 



void InitGL(void) 
{ 

	//set up misc opengl constants
	glClearColor(0,0,0,1); //background is black


	//enable zbuffering, backface culling, and normalizing
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	//glEnable(GL_NORMALIZE);

	GLfloat ambient[] = {.25,.25f,.25f,1};
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT,ambient);

	//use glcolor to set ambient and diffuse materials properties
	glColorMaterial( GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	//set the depth buffer parameters
	glClearDepth(1.0);
	glDepthFunc(GL_LESS);

	//set the shading model to smooth 
	glShadeModel(GL_SMOOTH);

	//default to bilenear perspective texture correction
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_FASTEST);

	glMatrixMode(GL_MODELVIEW);
} 



void IdleFunc(void) 
{ 
	theScene.Update();
	RenderScene();
} 

/* 
GLUT keyboard callback 

Params: 
key                the key pressed 
x                the coordinates of 
y                 the mouse pointer 
Returns: 
void 
*/ 
void KeyFunc(unsigned char key, int mx, int my) 
{ 
	//quit upon the user pressing q 
	InputEvent e = InputEvent();
	e.SetParams(InputEvent::EventType::KeyDown,Vector3((float)mx,(float)my,0),key);
	theScene.AddInputEvent(e);
}

void MotionFunc(int x, int y)
{
	InputEvent e = InputEvent();
	Vector3 v;
	if (LB)
	{
		v.Set((float)x-dx,0);
		v.Set((float)y-dy,1);
	}

	if (RB)
		v.Set((float)y-dy,2);

	e.SetParams(InputEvent::EventType::MouseDrag,v,0);
	theScene.AddInputEvent(e);
	dx = x;
	dy = y;
}

void MouseFunc(int button, int state, int x, int y)
{
	if (button == SDL_BUTTON_LEFT)
	{
		if (!RB && state == SDL_PRESSED)
		{
			dx = x;
			dy = y;
			LB = true;
		}
		else
			LB = false;
	}

	if (button == SDL_BUTTON_RIGHT)
	{
		if (!LB && state == SDL_PRESSED)
		{
			dx = x;
			dy = y;
			RB = true;
		}
		else
			RB = false;
	}

}

bool CheckEvents()
{
	/* This could go in a separate function */
	{
		SDL_Event event;
		while ( SDL_PollEvent(&event) ) 
		{
			switch(event.type)
			{
			case SDL_QUIT:
				{
					return false;
					break;
				}

			case SDL_KEYDOWN:
				{
					int x,y;
					SDL_GetMouseState(&x,&y);
					KeyFunc(event.key.keysym.sym,x,y);
					break;
				}
			case SDL_MOUSEMOTION:
				{
					MotionFunc(event.motion.x,event.motion.y);
					break;
				}

			case SDL_MOUSEBUTTONUP:
			case SDL_MOUSEBUTTONDOWN:
				{
					MouseFunc(event.button.button,event.button.state,event.button.x,event.button.y);
					break;
				}
			default:
				break;


			}
		}
	}


	return true;
}

int main(int argc, char **argv) 
{ 


	printf("Initializing SDL.\n");

	/* Initialize defaults, Video and Audio */
	if((SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO)==-1)) { 
		printf("Could not initialize SDL: %s.\n", SDL_GetError());
		exit(-1);
	}

	printf("SDL initialized.\n");


	// Tell SDL that the OpenGL rendering is going to be double buffered.
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 32 );
	SDL_GL_SetAttribute( SDL_GL_STENCIL_SIZE, 0 );
	SDL_GL_SetAttribute( SDL_GL_ACCUM_RED_SIZE, 0 );
	SDL_GL_SetAttribute( SDL_GL_ACCUM_GREEN_SIZE, 0 );
	SDL_GL_SetAttribute( SDL_GL_ACCUM_BLUE_SIZE, 0 );
	SDL_GL_SetAttribute( SDL_GL_ACCUM_ALPHA_SIZE, 0 );

	SDL_Surface *surface = NULL;
	if (fullScreen)
		surface =  SDL_SetVideoMode(XSIZE, YSIZE, 32, SDL_OPENGL | SDL_FULLSCREEN);
	else
		surface = SDL_SetVideoMode(XSIZE, YSIZE, 32, SDL_OPENGL);
	if (surface  == NULL ) {
		fprintf(stderr, "Unable to create OpenGL screen: %s\n", SDL_GetError());
		SDL_Quit();
		exit(2);
	}

	/* Set the title bar in environments that support it */
	SDL_WM_SetCaption("The Boxer", NULL);
	TimeStamp::SetTimeZero();
	InitGL();
	MB=LB=RB=false;

	srand(SDL_GetTicks());

	if (!theScene.Initialize())
		return 1;

	theScene.SetWindowSize(XSIZE,YSIZE);

	for(;;)
	{
		IdleFunc();
		if (!CheckEvents())
			break;
	}

	printf("Quiting SDL.\n");
	/* Shutdown all subsystems */
	SDL_Quit();
	printf("Quiting....\n");
	exit(0);

	return 0; 
} 







