#include <Font.h>
#include <SDL.h>
#include <SDL_Image.h>
#include <SDLimprc.h>
#include <windows.h>
#include <GL/gl.h> 

std::map<std::string,Font*> Font::fonts;

Font::Font()
{
	base =0;
	texture = 0;
	loaded = false;
}
Font::~Font()
{
	if (loaded)
	{
		glDeleteTextures(1,&texture);
		glDeleteLists(base,128);
	}
}

bool 
Font::LoadFont(const char * fname, const char * wname, const char * name, bool flip)
{

	if (fonts.find(name) != fonts.end())
	{
		printf("font already loaded\n");
		return false;
	}

	//load the data
	unsigned char * data = NULL;

	SDL_Surface *image;
	
	image=IMG_Load(fname);
	if(!image) 
	{
		printf("IMG_Load: %s\n", IMG_GetError());
		return false;
	}
	
	FILE * wf = fopen(wname,"rb");
	if (!wf)
	{
		printf("width file not found\n");
		return false;
	}
	
	Font *font = new(Font);
	
	
	data = ConvertImage(image,flip);	

	glGenTextures(1, &font->texture);		// Create The Texture
	glBindTexture(GL_TEXTURE_2D, font->texture); //select the texture
	glTexImage2D(GL_TEXTURE_2D, 0, 3, image->w, image->h, 0, GL_RGB, GL_UNSIGNED_BYTE, data); //load the data into the texture

	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);	// Linear Filtering
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);	// Linear Filtering
	
	float pixelwidth = 1.0f/image->w;
	float charwidth = 1.0f / 16;
	SDL_FreeSurface(image);
	delete data;


	//generate the display lists
	font->base = glGenLists(128);
	
	for (int i = 0; i < 128;i++)
	{
		unsigned char d;
		fread(&d,1,1,wf);
		float w = (charwidth - d*pixelwidth)/2.0f;

		float s = 1.0/16.0;
		int x = i % 16;
		int y = i / 16;

		glNewList(font->base+i,GL_COMPILE);
		glBegin(GL_QUADS);
		glColor3f(1,1,1);
		glNormal3f(0,0,1);
		
		glTexCoord2f(s*x+w,y*s);
		glVertex2f(0,0);
		
		glTexCoord2f(s*x+w,(y+1)*s);
		glVertex2f(0,1);
		
		glTexCoord2f((x+1)*s-w, (y+1)*s);
		glVertex2f((d*pixelwidth)/charwidth,1);
		
		glTexCoord2f((x+1)*s-w, y*s);
		glVertex2f((d*pixelwidth)/charwidth,0);
		glEnd();
		glTranslatef((d*pixelwidth)/charwidth,0,0);
		glEndList();
	}

	fonts[name] = font;

	font->loaded = true;
	fclose(wf);
	return true;
}

void
Font::UnLoadFont(const char * name)
{
	std::map<std::string,Font*>::iterator it = fonts.find(name);
	if (it == fonts.end())
		return;

	delete (it->second);
	fonts.erase(it);
}

void 
Font::Printf(const char* name, unsigned int scale, bool shift, const char *fmt, ...)
{
	std::map<std::string,Font*>::iterator it = fonts.find(name);
	if (it == fonts.end())
		return;

	Font * f = it->second;

	char text[256]; /* Holds our string */
    va_list ap;     /* Pointer to our list of elements */

    /* If there's no text, do nothing */
    if ( fmt == NULL )
	return;

    /* Parses The String For Variables */
    va_start( ap, fmt );
      /* Converts Symbols To Actual Numbers */
      vsprintf( text, fmt, ap );
    va_end( ap );

    /* Pushes the Display List Bits */
    glPushAttrib( GL_LIST_BIT );

    /* Sets base character to 32 */
    if (shift)
		glListBase( f->base - 32 +96);
	else
		glListBase( f->base - 32 );


	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, f->texture);	

    glPushMatrix();

	glScalef((float)scale,(float)scale,(float)scale);
	/* Draws the text */
    glCallLists( (GLsizei)strlen( text ), GL_UNSIGNED_BYTE, text );

    glPopMatrix();
	/* Pops the Display List Bits */
    glPopAttrib( );
	glDisable(GL_TEXTURE_2D);

}
